"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCompositedComponentClass = exports.generateCompositeComponent = void 0;
const tslib_1 = require("tslib");
/* eslint-disable react/prop-types */
const React = tslib_1.__importStar(require("react"));
const RenderWrapper_1 = require("../handlers/RenderWrapper");
const hooks_1 = require("../utils/hooks");
const index_1 = require("./index");
const cals_1 = require("@cloudbase/cals");
const global_api_1 = require("../app/global-api");
const lowcode_1 = require("../utils/adapter/lowcode");
const generate_1 = require("../utils/adapter/generate");
const adapter_1 = require("../utils/adapter");
const mobx_1 = require("mobx");
const json_schema_defaults_1 = tslib_1.__importDefault(require("json-schema-defaults"));
const utils_1 = require("../utils");
const weda_client_1 = require("@cloudbase/weda-client");
const datasource_1 = require("../utils/adapter/datasource");
const { createDataset, createStateDataSourceVar, generateParamsParser, setDatasetProfiles } = weda_client_1._WEDA_CLOUD_SDK;
function useLoad(defaultValue, callback, options = {}, deps) {
    const { looseError = false } = options;
    const [loaded, setLoaded] = React.useState(false);
    const [error, setError] = React.useState(null);
    const [data, setData] = React.useState(defaultValue);
    React.useEffect(() => {
        function run() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                try {
                    const lowcode = yield callback();
                    setData(lowcode);
                    setLoaded(true);
                    setError(null);
                }
                catch (e) {
                    if (!looseError) {
                        setError(e);
                    }
                    else {
                        console.log(e);
                    }
                    setLoaded(true);
                }
            });
        }
        run();
    }, deps);
    return { data, error, loaded, setData, setError, setLoaded };
}
function loadLowcodesDefult(sourceKey, comp, module, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { processCssUnit = 'rem', lowcodeContext } = options;
        const codeModulesMap = {
            $comp: (comp.resources || []).map((module) => {
                if (module.codeType === cals_1.ECodeType.THEME || module.codeType === cals_1.ECodeType.STYLE) {
                    return module;
                }
                const { code } = module, restModule = tslib_1.__rest(module, ["code"]);
                return Object.assign(Object.assign({}, restModule), { code: code.replace(/\$comp/g, 'this.$comp').replace(/\$w/g, 'this.$comp.__internal__.$w') });
            }),
        };
        const styleModulesMap = {
            $comp: {
                resources: codeModulesMap.$comp,
                processCssUnit,
                cssContextWrapper: getCompositedComponentClass({ module, component: comp.name }),
            },
        };
        const [styleCodesMap, { state, computedFuncs, lifeCycle, handler }] = yield Promise.all([
            (0, lowcode_1.generateStyleModuleMap)(styleModulesMap),
            (0, lowcode_1.generateLowcodeModules)({
                key: sourceKey,
                codeModulesMap,
                runtimeWindow: adapter_1.runtime,
                context: lowcodeContext,
            }).catch((e) => {
                e.message = `[低码模块加载失败] ${e.absoluteReqPath ? `[${e.absoluteReqPath}]` : ''} ${e.message}`;
                throw e;
            }),
        ]);
        const lowcode = {
            state: state.$comp || {},
            computed: computedFuncs.$comp || {},
            lifecycle: lifeCycle.$comp || {},
            handler: handler.$comp || {},
            style: styleCodesMap.$comp || [],
        };
        return lowcode;
    });
}
const COMPOSITECOMPONENT_DATASETPROFILE_MAP = {};
function generateCompositeComponent({ sourceKey, comp, module, componentsMap, componentActionsMap, componentsInfoMap, options, loadLowcodes = loadLowcodesDefult, }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!COMPOSITECOMPONENT_DATASETPROFILE_MAP[sourceKey] && comp.dataset) {
            setDatasetProfiles({
                [sourceKey]: (0, datasource_1.generateDynamicDataset)({ looseError: true }, comp.dataset),
            });
        }
        const lowcode = yield loadLowcodes(sourceKey, comp, module, options);
        const Component = React.forwardRef((props, ref) => {
            // const { data, children, $node } = useComponentProps(props as any, true);
            const schema = { items: comp.items, type: cals_1.EComponentType.COMPLEX, extra: {} };
            return (React.createElement(React.Fragment, null,
                React.createElement(index_1.ScopedStyle, { className: `${getCompositedComponentClass({ module, component: comp.name })}-style-tag`, schema: schema, options: options }),
                React.createElement(CompositeComponent, { ref: ref, sourceKey: sourceKey, comp: comp, props: props, generateLowcodeModule: () => lowcode, generateComponentsMap: () => componentsMap, generateActionsMap: () => componentActionsMap, generateContext: (ctx, comp) => {
                        const wrapperClass = getCompositedComponentClass({ module, component: comp.name });
                        const { widgets, componentSchema, dataBinds } = (0, generate_1.getComponentSchema)(ctx, schema, componentsInfoMap, wrapperClass, {
                            isComposite: true,
                            looseError: options.looseError,
                            processCssUnit: options.processCssUnit,
                            disablePageComponentInvoke: options.disablePageComponentInvoke,
                        });
                        return {
                            widgetContext: widgets,
                            componentSchema,
                            dataBinds,
                        };
                    }, options: options })));
        });
        Component.displayName = comp.name;
        return Component;
    });
}
exports.generateCompositeComponent = generateCompositeComponent;
function getCompositedComponentClass({ module, component }) {
    return `wa-comp-${module}-${component}`;
}
exports.getCompositedComponentClass = getCompositedComponentClass;
function useObserverDataRef(data, defaultData) {
    const ref = React.useRef((0, mobx_1.observable)(Object.assign(Object.assign({}, defaultData))));
    const [_, setForceUpate] = React.useReducer((data) => {
        return data + 1;
    }, 0);
    React.useEffect(() => {
        const dispose = (0, mobx_1.autorun)(() => {
            let update = false;
            for (const prop in data) {
                // 更新 propsData
                if (typeof data[prop] !== 'function') {
                    ref.current[prop] = data[prop];
                    update = true;
                }
            }
            if (update) {
                setForceUpate();
            }
        });
        return () => {
            dispose();
        };
    }, [data]);
    return { ref, key: _ };
}
const CompositeComponent = React.forwardRef(({ sourceKey, comp: _comp, props, generateComponentsMap, generateLowcodeModule, generateActionsMap, generateContext, options, }, ref) => {
    const { events, $node, data, className, style } = props;
    const { comp, eventFlows = [] } = React.useMemo(() => {
        const { listeners, eventFlows } = (0, cals_1.processListeners)(_comp.listeners || []);
        return { comp: Object.assign(Object.assign({}, _comp), { listeners }), eventFlows };
    }, [_comp]);
    const codeContextRef = React.useRef(null);
    const { ref: dataRef } = useObserverDataRef(data, (0, json_schema_defaults_1.default)(comp.data));
    const widgetsContextRef = React.useRef({});
    const globalCtxRef = React.useRef({});
    const $comp = codeContextRef.current;
    const [componentCodeContextInited, setcomponentCodeContextInited] = React.useState(false);
    const lowcodeRef = React.useRef(generateLowcodeModule());
    const lowcode = lowcodeRef.current;
    React.useEffect(() => {
        var _a;
        const $comp = (0, global_api_1.createCompApi)({
            id: comp.id,
            ref,
            state: (0, mobx_1.observable)(lowcode.state),
            computed: lowcode.computed,
            handler: lowcode.handler,
            props,
            propsData: dataRef.current,
            events,
            node: Object.assign($node || {}, {
                getConfig: () => comp.compConfig,
                getOwnerWidget: () => $node.ownerWidget,
                _debug: () => {
                    console.warn('仅用于调试，行为随时发生变化，不要用于生产环境。');
                    return $comp;
                },
            }),
            eventFlows,
        });
        const context = {
            $w: $comp.__internal__.$w,
            $app: global_api_1.app,
            app: global_api_1.app,
            $page: new Proxy({}, {
                get(target, p) {
                    var _a, _b;
                    return (_b = (_a = (0, weda_client_1.getWedaAPI)()) === null || _a === void 0 ? void 0 : _a.app.utils.getCurrentPage()) === null || _b === void 0 ? void 0 : _b[p];
                },
            }),
        };
        const dataset = createDataset(sourceKey, undefined, { appId: global_api_1.app === null || global_api_1.app === void 0 ? void 0 : global_api_1.app.id });
        createStateDataSourceVar(sourceKey, generateParamsParser(context));
        dataset.query = (0, generate_1.generateDatasetQuery)((_a = comp.dataset) === null || _a === void 0 ? void 0 : _a.query, context, {
            isComposite: false,
            looseError: true,
            useScopeWidgetData: true,
        });
        $comp.state.dataset = dataset;
        $comp.dataset = dataset;
        let stateSyncDisposes = (0, utils_1.watchAndSyncDatasetState2Local)(global_api_1.app.id, sourceKey, $comp.dataset.state);
        if (options === null || options === void 0 ? void 0 : options.processCssUnit) {
            $comp.__internal__.defaultCssUnit = options === null || options === void 0 ? void 0 : options.processCssUnit;
        }
        codeContextRef.current = $comp;
        setcomponentCodeContextInited(true);
        return () => {
            var _a, _b, _c, _d;
            (_c = (_b = (_a = lowcode.lifecycle.onAttached) === null || _a === void 0 ? void 0 : _a.bind) === null || _b === void 0 ? void 0 : _b.call(_a, codeContextRef.current)) === null || _c === void 0 ? void 0 : _c();
            Object.values(((_d = $comp === null || $comp === void 0 ? void 0 : $comp.dataset) === null || _d === void 0 ? void 0 : _d.query) || {}).forEach((query) => {
                var _a;
                (_a = query === null || query === void 0 ? void 0 : query.destroy) === null || _a === void 0 ? void 0 : _a.call(query);
            });
            stateSyncDisposes.forEach((dispose) => {
                dispose === null || dispose === void 0 ? void 0 : dispose();
            });
        };
    }, []);
    const { componentSchema, widgetsContext } = React.useMemo(() => {
        if (componentCodeContextInited) {
            globalCtxRef.current = {
                $comp,
                $app: global_api_1.app,
                actionsMap: (generateActionsMap === null || generateActionsMap === void 0 ? void 0 : generateActionsMap()) || {},
            };
            const globalCtx = globalCtxRef.current;
            let widgetsContext;
            let databinds;
            let componentSchema;
            if (generateContext) {
                let context = generateContext(globalCtx, comp);
                widgetsContext = context.widgetContext;
                databinds = context.dataBinds;
                componentSchema = context.componentSchema;
            }
            // widgets 内的 dataBinds 可能需要关联 widgets，需要重新执行 dataBinds
            $comp._dataBinds = databinds;
            return { componentSchema, widgetsContext };
        }
        return {};
    }, [componentCodeContextInited, comp]);
    widgetsContextRef.current = React.useMemo(() => {
        if (componentCodeContextInited) {
            const globalCtx = globalCtxRef.current;
            const { widgets, nodeMap } = (0, utils_1.createWidgets)({ apiContext: $comp, globalCtx }, widgetsContext, $comp._dataBinds || {}, {});
            Object.entries(widgets).forEach(([key, value]) => {
                $comp.widgets[key] = value;
                const list = !Array.isArray(value) ? [value] : value;
                list.forEach((item) => {
                    item.ownerWidget = $comp.node;
                    item.getOwnerWidget = () => $comp.node;
                });
            });
            setTimeout(utils_1.retryDataBinds);
            return { widgetProps: widgetsContext, nodeMap, widgets };
        }
        else {
            return widgetsContextRef.current;
        }
    }, [widgetsContext]);
    return (componentCodeContextInited && (React.createElement(CompositeComponentContent, { className: className, style: style, comp: comp, lowcode: lowcode, componentSchema: componentSchema, componentCodeContext: codeContextRef.current, virtualFields: generateComponentsMap() })));
});
CompositeComponent.displayName = 'CompositeComponent';
function CompositeComponentContent({ lowcode, componentSchema, componentCodeContext, virtualFields, className = '', style = {}, }) {
    const { attached } = React.useContext(hooks_1.OwnerComponentContext);
    const [mount, setMounted] = React.useState(false);
    React.useEffect(() => {
        setMounted(true);
        return () => {
            var _a, _b, _c;
            if (componentCodeContext.widgets._disposers) {
                componentCodeContext.widgets._disposers.forEach((disposer) => disposer());
                componentCodeContext.widgets._disposers = null;
            }
            (_c = (_b = (_a = lowcode === null || lowcode === void 0 ? void 0 : lowcode.lifecycle.onDetached) === null || _a === void 0 ? void 0 : _a.bind) === null || _b === void 0 ? void 0 : _b.call(_a, componentCodeContext)) === null || _c === void 0 ? void 0 : _c();
            componentCodeContext._instanceRef.current = undefined;
            setMounted(false);
        };
    }, []);
    React.useEffect(() => {
        var _a, _b, _c;
        if (attached) {
            (_c = (_b = (_a = lowcode.lifecycle.onReady) === null || _a === void 0 ? void 0 : _a.bind) === null || _b === void 0 ? void 0 : _b.call(_a, componentCodeContext)) === null || _c === void 0 ? void 0 : _c();
        }
    }, [attached]);
    const ownerComponentContext = React.useMemo(() => {
        return { attached: mount };
    }, [mount]);
    return (React.createElement(hooks_1.OwnerComponentContext.Provider, { value: ownerComponentContext },
        React.createElement(RenderWrapper_1.AppRender, { className: className, style: style, virtualFields: virtualFields, componentSchema: componentSchema, codeContext: componentCodeContext })));
}
//# sourceMappingURL=comositeComponent.js.map