"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isRepeaterWidget = exports.mergeDependencies = exports.loadMockDependencies = exports.loadRuntimeDependencies = exports.loadExternalResources = exports.WrapComponentFieldWithPropsHook = exports.componentMetaFilter = exports.formatDependiencesMap = exports.loadDependiences = exports.loadDependencies = exports.injectExternals = exports.upsertFavicon = void 0;
const tslib_1 = require("tslib");
/* eslint-disable react/display-name */
const React = tslib_1.__importStar(require("react"));
const ReactDom = tslib_1.__importStar(require("react-dom"));
const server_1 = tslib_1.__importDefault(require("react-dom/server"));
const MobxReactLite = tslib_1.__importStar(require("mobx-react-lite"));
const Mobx = tslib_1.__importStar(require("mobx"));
const lodash_es_1 = require("lodash-es");
const History = tslib_1.__importStar(require("history"));
const ReactErrorBoundary = tslib_1.__importStar(require("react-error-boundary"));
const cals_1 = require("@cloudbase/cals");
const hooks_1 = require("../hooks");
const index_1 = require("./index");
const constants_1 = require("../constants");
const WedaClientSDK = tslib_1.__importStar(require("@cloudbase/weda-client"));
const UMD_STATUS_MAP = {};
function loadUmdModule(jsUrl, umdModuleName, targetDoc = document, cache = false, module = true) {
    var _a;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const win = targetDoc.defaultView;
        if (cache) {
            if (module && Object.prototype.hasOwnProperty.call(win, umdModuleName)) {
                return win[umdModuleName];
            }
            if (!module) {
                if ((_a = UMD_STATUS_MAP[jsUrl]) === null || _a === void 0 ? void 0 : _a.promise) {
                    return UMD_STATUS_MAP[jsUrl].promise;
                }
                let scripts = Array.from(targetDoc.body.querySelectorAll('script'));
                if (scripts.find((item) => (item === null || item === void 0 ? void 0 : item.className) === umdModuleName)) {
                    return null;
                }
            }
        }
        const sign = Date.now();
        UMD_STATUS_MAP[jsUrl] = {
            sign,
            promise: new Promise((resolve, reject) => {
                const script = targetDoc.createElement('script');
                script.setAttribute('src', jsUrl);
                script.setAttribute('class', umdModuleName);
                script.addEventListener('load', () => {
                    if (module) {
                        if (Object.prototype.hasOwnProperty.call(win, umdModuleName)) {
                            UMD_STATUS_MAP[jsUrl] = {
                                loading: false,
                                data: win[umdModuleName],
                                error: null,
                            };
                            return resolve(win[umdModuleName]);
                        }
                        else {
                            const error = new Error(`Fail to load UMD module ${umdModuleName} from [${jsUrl}].`);
                            UMD_STATUS_MAP[jsUrl] = {
                                loading: false,
                                data: null,
                                error,
                            };
                            return reject(error);
                        }
                    }
                    else {
                        UMD_STATUS_MAP[jsUrl] = {
                            loading: false,
                            data: null,
                            error: null,
                        };
                        return resolve(null);
                    }
                });
                script.addEventListener('error', (e) => {
                    const error = new Error(`main bundle [${umdModuleName}] load failed from [${jsUrl}]: ${(e === null || e === void 0 ? void 0 : e.message) || ''}`);
                    reject(error);
                });
                targetDoc.body.appendChild(script);
            }).then((data) => {
                if (UMD_STATUS_MAP[jsUrl] && UMD_STATUS_MAP[jsUrl].sign === sign) {
                    UMD_STATUS_MAP[jsUrl].data = data;
                }
                return data;
            }, (e) => {
                if (UMD_STATUS_MAP[jsUrl].sign === sign) {
                    /**
                     * 清空缓存
                     */
                    UMD_STATUS_MAP[jsUrl] = undefined;
                }
                throw e;
            }),
        };
        return UMD_STATUS_MAP[jsUrl].promise;
    });
}
function loadCssModule(url, bundleName, entryDoc = document, cache = false, module = true) {
    if (!url) {
        return Promise.resolve();
    }
    return new Promise((resolve) => {
        const className = module ? `@weda-materials-style-${bundleName}` : bundleName;
        if (cache) {
            let links = Array.from(entryDoc.head.querySelectorAll('link'));
            if (links.find((item) => (item === null || item === void 0 ? void 0 : item.className) === className)) {
                return resolve();
            }
        }
        const link = entryDoc.createElement('link');
        link.type = 'text/css';
        link.rel = 'stylesheet';
        link.className = className;
        link.href = url;
        link.addEventListener('load', () => resolve());
        link.addEventListener('error', (e) => {
            console.warn(`Load [${bundleName}] meta style failed, skip...`, e);
            resolve();
        });
        entryDoc.head.appendChild(link);
    });
}
function upsertFavicon(favicon, app, entryDoc = document) {
    if (!favicon) {
        return;
    }
    const { __internal__ = {} } = app || {};
    const { resolveStaticResourceUrl } = __internal__ || {};
    const href = resolveStaticResourceUrl(favicon);
    let links = Array.from(entryDoc.head.querySelectorAll('link[rel~="icon"]'));
    if (links.length) {
        links.forEach((link) => {
            link.href = href;
        });
    }
    else {
        const link = entryDoc.createElement('link');
        link.type = 'image/x-icon';
        link.rel = 'shortcut icon';
        link.href = href;
        entryDoc.head.appendChild(link);
    }
}
exports.upsertFavicon = upsertFavicon;
function injectExternals(runtime = index_1.runtime) {
    const externals = {
        // 兼容配置
        react: runtime.React || React,
        React: runtime.React || React,
        ReactDOM: runtime.ReactDOM || ReactDom,
        ReactDom: runtime.ReactDOM || ReactDom,
        ReactDomServer: server_1.default,
        Mobx: runtime.mobx || Mobx,
        'mobx-react-lite': MobxReactLite,
        MobxReactLite,
        LodashGet: lodash_es_1.get,
        LodashRemove: lodash_es_1.remove,
        LodashSet: lodash_es_1.set,
        ReactErrorBoundary,
        History,
        WedaClientSDK: runtime.WedaClientSDK || WedaClientSDK,
        WedaAppContext: runtime.WedaAppContext || (WedaClientSDK === null || WedaClientSDK === void 0 ? void 0 : WedaClientSDK.AppContext),
    };
    for (const key in externals) {
        runtime[key] = externals[key];
    }
}
exports.injectExternals = injectExternals;
function loadDependencies(data, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const controlPromises = [];
        const componentPromises = [];
        const cssPromises = [];
        const dependencies = Array.isArray(data) ? data : data.dependencies;
        const runtime = !Array.isArray(data) && data.runtime ? data.runtime : index_1.runtime;
        const cache = typeof options === 'boolean' ? options : (options === null || options === void 0 ? void 0 : options.cache) || false;
        dependencies.forEach((item) => {
            if (!item.isComposite) {
                controlPromises.push(loadUmdModule(item.controlJsPath || item.mainJsPath.replace(/\.main\.js/, '.control.js'), `@weapps-materials-control-${item.name}`, runtime.document, cache)
                    .then((res) => {
                    res.name = item.name;
                    res.version = item.version;
                    return res;
                })
                    .catch((e) => {
                    console.warn(e);
                    return null;
                }));
            }
            componentPromises.push(loadUmdModule(item.mainJsPath, `@weapps-materials-main-${item.name}`, runtime.document, cache).then((res) => {
                res.name = item.name;
                res.isComposite = item.isComposite;
                return res;
            }, (e) => {
                console.warn(e);
                return null;
            }));
            cssPromises.push(loadCssModule(item.mainCssPath, item.name, runtime.document, cache));
        });
        const [controls, components] = yield Promise.all([controlPromises, componentPromises, cssPromises].map((arr) => Promise.all(arr)));
        let res = formatDependiencesMap(components, controls);
        const runtimeConfig = yield loadRuntimeDependencies();
        return mergeDependencies([res, runtimeConfig]);
    });
}
exports.loadDependencies = loadDependencies;
exports.loadDependiences = loadDependencies;
function formatDependiencesMap(components, controls) {
    const componentsMap = {};
    const componentActionsMap = {};
    components
        .filter((item) => !!item)
        .forEach((lib) => {
        var _a, _b;
        let { schemaVersion = '' } = lib;
        let isPlainProps = true;
        if (lib.isComposite ||
            isNaN(Number((_a = schemaVersion.split('.')) === null || _a === void 0 ? void 0 : _a[0])) ||
            Number((_b = schemaVersion.split('.')) === null || _b === void 0 ? void 0 : _b[0]) < 3) {
            isPlainProps = false;
        }
        for (const componentName in lib.components || {}) {
            const sourceKey = `${lib.name}:${componentName}`;
            componentsMap[sourceKey] = WrapComponentFieldWithPropsHook({
                Field: lib.components[componentName],
                isPlainProps,
                sourceKey,
            });
        }
        for (const actionName in lib.actions) {
            componentActionsMap[`${lib.name}:${actionName}`] = lib.actions[actionName];
        }
    });
    const controlMap = controls
        .filter((item) => !!item)
        .reduce((map, lib) => {
        var _a, _b;
        map[`${lib.name}@${lib.version}`] = Object.assign(Object.assign({}, lib), { components: ((_b = (_a = lib === null || lib === void 0 ? void 0 : lib.components) === null || _a === void 0 ? void 0 : _a.map) === null || _b === void 0 ? void 0 : _b.call(_a, (item) => {
                /**
                 * 序列化的 pick 修改需要同步修改 generator
                 */
                const { name, dataForm, properties, emitEvents, events, meta, compConfig, inputProps, syncProps, isContainer, } = item;
                return {
                    name,
                    dataForm: Object.entries(dataForm || {}).reduce((map, [key, config]) => {
                        const { type, ['x-slot-scope']: xSlotScope, ['x-slot-directives']: xSlotDirectives, syncProp, inputProp, } = config;
                        const filteredDataForm = {
                            type,
                            'x-slot-scope': xSlotScope,
                            'x-slot-directives': xSlotDirectives,
                            syncProp,
                            inputProp,
                        };
                        map[key] = filteredDataForm;
                        return map;
                    }, {}),
                    properties: properties
                        ? Object.assign(Object.assign({}, properties), { properties: Object.entries(properties.properties || {}).reduce((map, [key, config]) => {
                                const { type, title } = config;
                                map[key] = {
                                    type,
                                    title,
                                };
                                return map;
                            }, {}) }) : undefined,
                    emitEvents,
                    events,
                    meta: componentMetaFilter(meta),
                    compConfig,
                    inputProps,
                    syncProps,
                    isContainer,
                };
            })) || [] });
        return map;
    }, {});
    // console.log('controlMap', controlMap);
    // console.log('componentsMap', componentsMap);
    return {
        controlMap,
        componentsMap,
        componentActionsMap,
    };
}
exports.formatDependiencesMap = formatDependiencesMap;
function componentMetaFilter(meta) {
    const { name: metaName, title, platform, shortcut, previewOperation } = meta || {};
    return meta ? { name: metaName, title, platform, shortcut, previewOperation } : undefined;
}
exports.componentMetaFilter = componentMetaFilter;
function WrapComponentFieldWithPropsHook({ Field, isPlainProps, sourceKey, }) {
    const name = Field.displayName || Field.name || 'Field';
    const processWrapper = (props, ref) => {
        const processedProps = (0, hooks_1.useComponentProps)(props, isPlainProps);
        return React.createElement(Field, Object.assign({}, processedProps, { ref: ref }));
    };
    processWrapper.displayName = `ForwordRef(${name})`;
    return React.forwardRef(processWrapper);
}
exports.WrapComponentFieldWithPropsHook = WrapComponentFieldWithPropsHook;
function loadExternalResources({ externalResources = [], runtime = index_1.runtime, }, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const scriptPromises = [];
        const cssPromises = [];
        const cache = options === null || options === void 0 ? void 0 : options.cache;
        for (let externalResource of externalResources) {
            switch (externalResource.type) {
                case cals_1.EExternalResourceType.JSUrl: {
                    scriptPromises.push(loadUmdModule(externalResource.url, externalResource.url, runtime.document, cache, false).catch((e) => {
                        console.warn(e);
                        return null;
                    }));
                    break;
                }
                case cals_1.EExternalResourceType.CSSUrl: {
                    cssPromises.push(loadCssModule(externalResource.url, externalResource.url, runtime.document, cache, false));
                    break;
                }
            }
        }
        yield Promise.all([...scriptPromises, ...cssPromises]);
    });
}
exports.loadExternalResources = loadExternalResources;
function loadRuntimeDependencies() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return {
            controlMap: {},
            componentsMap: {
                [`${constants_1.REPEATER.MODULE_NAME}:${constants_1.REPEATER.REPEATER_NAME}`]: WrapComponentFieldWithPropsHook({
                    Field: Repeater,
                    isPlainProps: true,
                }),
                [`${constants_1.REPEATER.MODULE_NAME}:${constants_1.REPEATER.REPEATER_ITEM_NAME}`]: WrapComponentFieldWithPropsHook({
                    Field: Block,
                    isPlainProps: true,
                }),
            },
            componentActionsMap: {},
        };
    });
}
exports.loadRuntimeDependencies = loadRuntimeDependencies;
function useSetWidgetApi(init, deps, // deps 要在第二个参数，否则linter会失效
ref) {
    React.useLayoutEffect(() => {
        let r;
        if (typeof init === 'function') {
            r = init();
        }
        else {
            r = init;
        }
        if (typeof ref === 'function') {
            ref(r);
        }
        else if (typeof ref === 'object' && ref !== null) {
            // ref.current = r; 这样会导致 ref.current 整个变化， 假如 ref.current = {pageSize: 1}
            // ref.current = {pageSize: 1} 会导致 query重复请求
            if (Object.prototype.toString.call(ref.current) !== '[object Object]') {
                ref.current = {};
            }
            Object.assign(ref.current, r);
        }
    }, deps);
}
function loadMockDependencies() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return {
            controlMap: {},
            componentsMap: {
            // ['gsd-h5-react:WdText']: WrapComponentFieldWithPropsHook({
            //   Field: MobxReactLite.observer(
            //     (props: any) => {
            //       // if (props.id === 'text122' || props.id === 'text2') {
            //       // useTraceUpdate(props);
            //       // console.log(`>>>**${props.id}`, props.level);
            //       // Mobx.trace(true);
            //       // }
            //       return (
            //         <p
            //           className={props.level}
            //           id={props.id}
            //           // eslint-disable-next-line react/jsx-props-no-spreading
            //         >
            //           {props.text}
            //         </p>
            //       );
            //     },
            //     { forwardRef: true },
            //   ),
            //   isPlainProps: true,
            //   sourceKey: 'gsd-h5-react:WdText',
            // }),
            // ['gsd-h5-react:Container']: WrapComponentFieldWithPropsHook({
            //   Field: MobxReactLite.observer(
            //     function Container(props: any, ref) {
            //       const { children, title, events, className, style, id, data } = props;
            //       if (id === 'container116' || id === 'container2') {
            //         useTraceUpdate(props);
            //       }
            //       useSetWidgetApi(
            //         () => {
            //           return {
            //             data,
            //           };
            //         },
            //         [data],
            //         ref,
            //       );
            //       return (
            //         <div
            //           role="container"
            //           id={id}
            //           style={style}
            //           className={classNames({ 'weda-ui': true, [className]: className })}
            //           title={title}
            //         >
            //           {children}
            //         </div>
            //       );
            //     },
            //     { forwardRef: true },
            //   ),
            //   isPlainProps: true,
            // }),
            // ['weda_adamsyu:Slot']: MobxReactLite.observer(
            //   (props: any) => {
            //     console.log('slot component props>>>>>>>>>>>>>>>>>>', props);
            //     const { id, text, events, compositeParent, data } = props;
            //     const { name } = data;
            //     if (!compositeParent) {
            //       console.warn('Slot组件只能用于实现复合组件');
            //       // 兼容编辑器点击，可选中 Slot 组件
            //       return <div>{`Slot - ${name}`}</div>;
            //     }
            //     return compositeParent.props.data[name] || null;
            //   },
            //   { forwardRef: true },
            // ),
            // eslint-disable-next-line react/display-name
            // [`${OFFICIAL_LIB_KEY}:WdText`]: React.forwardRef((props: React.PropsWithChildren<{}>, ref) => {
            //   // eslint-disable-next-line react/prop-types
            //   const processedProps = useComponentProps(props as any, true);
            //   return <DebugText {...processedProps} ref={ref} />;
            // }),
            },
            componentActionsMap: {},
        };
    });
}
exports.loadMockDependencies = loadMockDependencies;
const Repeater = React.forwardRef((props, ref) => {
    const { id, data, children, $node } = props;
    // if (id === 'repeater120' || id === 'repeater1') {
    //   useTraceUpdate(props);
    // }
    React.useImperativeHandle(ref, () => {
        return {
            data,
            items: $node.items,
        };
    }, [data, $node.children]);
    return typeof children === 'function' ? children() : children;
});
Repeater.displayName = constants_1.REPEATER.REPEATER_NAME;
const Block = React.forwardRef((props, ref) => {
    // eslint-disable-next-line react/prop-types
    const { children } = props;
    return typeof children === 'function' ? children() : children;
});
Block.displayName = 'Block';
// eslint-disable-next-line react/display-name
const DebugText = React.forwardRef((props, ref) => {
    // eslint-disable-next-line react/prop-types
    const { id, text, events, className, level = 'body-default', overflow = false, maxLines, space = true, tips = true, userSelect = false, contenteditable, onInput, onBlur, } = props;
    // const [value, setValue] = React.useState([]);
    React.useImperativeHandle(ref, () => {
        return {
            text,
            level,
            overflow,
            maxLines,
            space,
            userSelect,
            tips,
        };
    }, [text, level, overflow, maxLines, space, userSelect, tips]);
    // React.useEffect(() => {
    //   setInterval(() => {
    //     setValue(String(Math.random()));
    //     console.log('>>>>>>>>>>>>>>.');
    //     (ref as any).current.text = String(Math.random());
    //   }, 4000);
    // }, []);
    // const dispose = React.useMemo(() => {
    //   return Mobx.autorun((r) => {
    //     // r.trace();
    //     console.log('----------------');
    //     setValue(window?.$w?.wedaContext?.editorPlatforms);
    //   });
    // }, []);
    // React.useEffect(() => {
    //   return () => {
    //     dispose();
    //   };
    // }, []);
    return (React.createElement("p", { id: id, onClick: (e) => events.tap({}, { originEvent: e }), "data-testid": "text" }, String(text)));
});
function mergeDependencies(list) {
    const keys = ['controlMap', 'componentsMap', 'componentActionsMap'];
    return list.reduce((map, dependenceMap) => {
        for (const key of keys) {
            map[key] = Object.assign(Object.assign({}, map[key]), dependenceMap[key]);
        }
        return map;
    }, {
        controlMap: {},
        componentsMap: {},
        componentActionsMap: {},
    });
}
exports.mergeDependencies = mergeDependencies;
function isRepeaterWidget(w, repeaterComponentName /* 'Repeater' | 'RepeaterItem' */) {
    var _a;
    const { componentType } = ((_a = w === null || w === void 0 ? void 0 : w.getConfig) === null || _a === void 0 ? void 0 : _a.call(w)) || {};
    if (componentType === repeaterComponentName ||
        (w === null || w === void 0 ? void 0 : w.widgetType) === `${constants_1.REPEATER.MODULE_NAME}:${repeaterComponentName}` ||
        (w === null || w === void 0 ? void 0 : w.widgetType) === `${constants_1.REPEATER.DEBUG_MODULE_NAME}:${repeaterComponentName}`) {
        return true;
    }
}
exports.isRepeaterWidget = isRepeaterWidget;
//# sourceMappingURL=dependencies.js.map