import { kebabCase } from './common';
import { uniq as _uniq, map as _map } from 'lodash-es';
import { IPlatformComponent, PostcssRpx2Calc } from '@cloudbase/cals';
import { LCDS } from '@cloudbase/cals';
import { CALC_RPX_ROOT_VAL_NAME, postcssImport } from './index';

export function toCssText(style, className = '.some-class-name') {
  const attrText = Object.keys(style)
    .map((key) => {
      const value = style[key];
      return `${kebabCase(key)}: ${value};`;
    })
    .join('\n');
  return `${className} { ${attrText} }\n`;
}

export async function generateScopedStyleText(
  schema: IPlatformComponent,
  options: {
    enablePageRootId: boolean;
  },
) {
  const cssTextList = [];

  async function traverse(item: IPlatformComponent) {
    if (item.scopedStyle && item.id) {
      let cssText = item.scopedStyle
        // 移除 `/* ... */` 注释
        .replace(/\/\*(?:[\s\S]*?)\*\//g, '')
        // 移除换行及其前后的空白符
        .replace(/\s*\n\s*/g, '');

      if (options?.enablePageRootId) {
        // 添加 `#wd-page-root` 前缀以增加样式权重
        cssText = cssText.replace(/(:scope.*?[,{])/g, '\n#wd-page-root $1');
      }

      cssText = cssText
        // 替换 `:scope`
        .replace(/:scope/g, `.wd-comp-id-${item.id}`)
        // `: ` => `:`
        // `, ` => `,`
        // ` {` => `{`
        .replace(/\s*([:,{])\s*/g, '$1')
        // `;}` => `}`
        .replace(/;\s*}/g, '}');

      if (cssText.indexOf('rpx') !== -1) {
        const postcss = await postcssImport();
        const { css } = await postcss([PostcssRpx2Calc({ root: `var(${CALC_RPX_ROOT_VAL_NAME})` })]).process(cssText);
        cssText = css;
      }
      cssTextList.push(cssText.trim());
    }
    await Promise.all(item.items?.map?.((childItem) => traverse(childItem)) || []);
  }

  await traverse(schema);
  return cssTextList.join('\n');
}

export function mergeCssVarConfig(cssVarConfig = {}, platform: LCDS.XPlatform = 'PCWEB'): Record<string, string> {
  let merged = { ...(cssVarConfig as any)?._ };
  let list = [platform];
  switch (platform) {
    case 'PCWEB': {
      list = ['WEB', 'PCWEB'];
      break;
    }
    case 'MOBILEWEB': {
      list = ['WEB', 'MOBILE', 'MOBILEWEB'];
      break;
    }
    case 'MP': {
      list = ['MOBILE', 'MP'];
      break;
    }
    case 'WEB':
    case 'MOBILE': {
      list = [platform];
    }
  }
  for (let key of list) {
    if (cssVarConfig[key]) {
      merged = {
        ...merged,
        ...cssVarConfig[key],
      };
    }
  }
  return merged;
}
